@@ -1,6 +1,11 @@
-use super::{Linear, ModuleCopy, NNModule, LayerNorm, Embedding};
+use super::{Embedding, LayerNorm, Linear, ModuleCopy, NNModule, WeightCopyError};
 use tch::{nn, Device, IndexOp, Kind, Tensor};
 
+/// Different types of positional encoding for Transformers
+pub enum PositionalEncoding {
+    Learned,
+    Sinusoidal,
+}
 
 /// The most basic dot-product self attention with an optional causal mask
 #[derive(Debug)]
@@ -86,17 +91,18 @@ impl NNModule for SelfAttention {
 }
 
 impl ModuleCopy for SelfAttention {
-    fn copy(&mut self, source: &Self) {
-        self.key.copy(&source.key);
-        self.query.copy(&source.query);
-        self.value.copy(&source.value);
-        self.proj.copy(&source.proj);
+    fn copy(&mut self, source: &Self) -> Result<(), WeightCopyError> {
+        self.key.copy(&source.key)?;
+        self.query.copy(&source.query)?;
+        self.value.copy(&source.value)?;
+        self.proj.copy(&source.proj)?;
+        Ok(())
     }
 }
 
 /// A basic transformer encoder block
 #[derive(Debug)]
-struct TransformerBlock {
+pub struct TransformerBlock {
     norm1: LayerNorm,
     norm2: LayerNorm,
     attn: SelfAttention,
@@ -108,6 +114,7 @@ struct TransformerBlock {
 
 impl TransformerBlock {
     fn new(p: &nn::Path, n_embd: i64, n_head: i64, dropout: f64, causal_mask: bool) -> Self {
+        assert!(n_embd % n_head == 0, "Embedding size ({}) must be divisible by number of heads ({})!", n_embd, n_head);
         TransformerBlock {
             norm1: LayerNorm::new(p / "ln1", vec![n_embd]),
             norm2: LayerNorm::new(p / "ln2", vec![n_embd]),
@@ -143,12 +150,12 @@ impl NNModule for TransformerBlock {
 }
 
 impl ModuleCopy for TransformerBlock {
-    fn copy(&mut self, source: &Self) {
-        self.attn.copy(&source.attn);
-        self.norm1.copy(&source.norm1);
-        self.norm2.copy(&source.norm2);
-        self.linear1.copy(&source.linear1);
-        self.linear2.copy(&source.linear2);
+    fn copy(&mut self, source: &Self) -> Result<(), WeightCopyError> {
+        self.attn.copy(&source.attn)?;
+        self.norm1.copy(&source.norm1)?;
+        self.norm2.copy(&source.norm2)?;
+        self.linear1.copy(&source.linear1)?;
+        self.linear2.copy(&source.linear2)
     }
 }
 
@@ -156,7 +163,7 @@ impl ModuleCopy for TransformerBlock {
 #[derive(Debug)]
 pub struct TransformerEncoder {
     token_embedding: Embedding,
-    position_embedding: Tensor,
+    position_embedding: LocalPositionalEncoding,
     layernorm: LayerNorm,
     blocks: Vec<TransformerBlock>,
     dropout: f64,
@@ -165,16 +172,35 @@ pub struct TransformerEncoder {
     train: bool,
 }
 
+/// An enum for positional encoding which conains a tensor (only used internally)
+#[derive(Debug, PartialEq)]
+enum LocalPositionalEncoding {
+    Learned(Tensor),
+    Sinusoidal(Tensor),
+}
+
 impl TransformerEncoder {
     #[allow(clippy::too_many_arguments)]
-    pub fn new(p: &nn::Path, n_embd: i64, n_head: i64, n_layers: i64, vocab_size: i64, max_len: i64, dropout: f64, causal_mask: bool) -> Self {
+    pub fn new(p: &nn::Path, n_embd: i64, n_head: i64, n_layers: i64, vocab_size: i64, positional_encoding: PositionalEncoding, max_len: i64, dropout: f64, causal_mask: bool) -> Self {
         TransformerEncoder {
             token_embedding: Embedding::new(
                 p / "tok_emb",
                 vocab_size,
                 n_embd,
             ),
-            position_embedding: p.zeros("pos_emb", &[1, max_len, n_embd]),
+            position_embedding: match positional_encoding {
+                PositionalEncoding::Learned => LocalPositionalEncoding::Learned(p.randn("pos_emb", &[1, max_len, n_embd], 0., 0.5)),
+                PositionalEncoding::Sinusoidal => {
+                    // Build the sinusoidal vector (This is based on an online implementation here: https://towardsdatascience.com/how-to-code-the-transformer-in-pytorch-24db27c8f9ec#d554
+                    let mut pe = vec![vec![0.; n_embd as usize]; max_len as usize];
+                    for pos in 0..max_len as usize {
+                        for i in 0..n_embd as usize {
+                            pe[pos][i] = (pos as f64 / f64::powf(10000., (2.*i as f64) / n_embd as f64)).sin();
+                        }
+                    }
+                    LocalPositionalEncoding::Sinusoidal(Tensor::of_slice2(&pe).to_kind(Kind::Float).to(p.device())) // Doesn't need to be a variable, we aren't tracking it's gradients
+                }
+            },
             layernorm: LayerNorm::new(p / "ln_f", vec![n_embd]),
             blocks: {
                 //let p = &p.set_group(0);
@@ -193,8 +219,13 @@ impl TransformerEncoder {
 
     pub fn forward_no_embed(&self, xs: &Tensor) -> Tensor {
         // xs shape: (batch size, seq len, n_embd)
-        let (_, sz_t, _) = xs.size3().unwrap();
-        let pos_emb = self.position_embedding.i((.., ..sz_t, ..));
+        let (batch_size, sz_t, _) = xs.size3().unwrap();
+        let pos_emb = match &self.position_embedding {
+            LocalPositionalEncoding::Learned(l) => l.i((.., ..sz_t, ..)).repeat(&[batch_size, 1, 1]),
+            LocalPositionalEncoding::Sinusoidal(pe) => {
+                pe.i(..sz_t).repeat(&[batch_size, 1, 1])
+            }
+        };
         let mut x = (xs + pos_emb)
             .dropout(self.dropout, self.train);
         // Run through transformer blocks
@@ -226,10 +257,15 @@ impl NNModule for TransformerEncoder {
 
     fn forward(&self, x: &tch::Tensor) -> tch::Tensor {
         // x shape: (batch size, seq len)
-        let (_, sz_t) = x.size2().unwrap();
+        let (batch_size, sz_t) = x.size2().unwrap();
         // Run through embeddings
         let tok_emb = self.token_embedding.forward(x);
-        let pos_emb = self.position_embedding.i((.., ..sz_t, ..));
+        let pos_emb = match &self.position_embedding {
+            LocalPositionalEncoding::Learned(l) => l.i((.., ..sz_t, ..)).repeat(&[batch_size, 1, 1]),
+            LocalPositionalEncoding::Sinusoidal(pe) => {
+                pe.i(..sz_t).repeat(&[batch_size, 1, 1])
+            }
+        };
         let x = (tok_emb + pos_emb)
             .dropout(self.dropout, self.train);
         // Run through transformer blocks
@@ -245,16 +281,32 @@ impl NNModule for TransformerEncoder {
 }
 
 impl ModuleCopy for TransformerEncoder {
-    fn copy(&mut self, source: &Self) {
+    fn copy(&mut self, source: &Self) -> Result<(), WeightCopyError> {
         assert_eq!(self.blocks.len(), source.blocks.len());
-        self.token_embedding.copy(&source.token_embedding);
-        tch::no_grad(|| {
-            self.position_embedding.copy_(&source.position_embedding);
-        });
-        self.layernorm.copy(&source.layernorm);
-        for i in 0..self.blocks.len() {
-            self.blocks[i].copy(&source.blocks[i]);
+        self.token_embedding.copy(&source.token_embedding)?;
+        let (source_tensor, source_kind) = match &source.position_embedding {
+            LocalPositionalEncoding::Learned(t) => (t, true),
+            LocalPositionalEncoding::Sinusoidal(t) => (t, false),
+        };
+        let (target_tensor, target_kind) = match &mut self.position_embedding {
+            LocalPositionalEncoding::Learned(t) => (t, true),
+            LocalPositionalEncoding::Sinusoidal(t) => (t, false),
+        };
+        if source_kind != target_kind {
+            return Err(WeightCopyError::Other("Positional Encodings are of wrong type!".to_string()));
         }
+        if source_tensor.size() != target_tensor.size() {
+            return Err(WeightCopyError::SizeMismatch);
+        }
+
+        tch::no_grad(|| {
+            target_tensor.copy_(source_tensor);
+        });
+        self.layernorm.copy(&source.layernorm)?;
+        for i in 0..self.blocks.len() {
+            self.blocks[i].copy(&source.blocks[i])?;
+        }
+        Ok(())
     }
 }
 
@@ -269,9 +321,9 @@ pub struct TransformerAggregator {
 
 impl TransformerAggregator {
     #[allow(clippy::too_many_arguments)]
-    pub fn new(p: &nn::Path, n_embd: i64, n_head: i64, n_layers: i64, aggregation_size: i64, vocab_size: i64, max_len: i64, dropout: f64) -> Self {
+    pub fn new(p: &nn::Path, n_embd: i64, n_head: i64, n_layers: i64, aggregation_size: i64, vocab_size: i64, positional_encoding: PositionalEncoding, max_len: i64, dropout: f64) -> Self {
         TransformerAggregator {
-            encoder: TransformerEncoder::new(&(p / "encoder"), n_embd, n_head, n_layers, vocab_size, max_len, dropout, false),
+            encoder: TransformerEncoder::new(&(p / "encoder"), n_embd, n_head, n_layers, vocab_size, positional_encoding, max_len, dropout, false),
             head: Linear::new(&(p / "aggregation_head"), n_embd, aggregation_size),
             aggregation_embedding: p.randn("aggregation_vector", &[n_embd], 0.0, 0.2),
         }
@@ -312,12 +364,15 @@ impl NNModule for TransformerAggregator {
 }
 
 impl ModuleCopy for TransformerAggregator {
-    fn copy(&mut self, source: &Self) {
-        self.encoder.copy(&source.encoder);
+    fn copy(&mut self, source: &Self) -> Result<(), WeightCopyError> {
+        self.encoder.copy(&source.encoder)?;
+        if self.aggregation_embedding.size() != source.aggregation_embedding.size() {
+            return Err(WeightCopyError::SizeMismatch);
+        }
         tch::no_grad(|| {
             self.aggregation_embedding.copy_(&source.aggregation_embedding);
         });
-        self.head.copy(&source.head);
+        self.head.copy(&source.head)
     }
 }
 
@@ -335,9 +390,9 @@ unsafe impl Send for LanguageModel {}
 unsafe impl Sync for LanguageModel {}
 
 impl LanguageModel {
-    pub fn new(p: &nn::Path, n_embd: i64, n_head: i64, n_layers: i64, vocab_size: i64, max_len: i64, dropout: f64) -> Self {
+    pub fn new(p: &nn::Path, n_embd: i64, n_head: i64, n_layers: i64, vocab_size: i64, positional_encoding: PositionalEncoding, max_len: i64, dropout: f64) -> Self {
         LanguageModel {
-            transformer: TransformerEncoder::new(&(p / "transformer"), n_embd, n_head, n_layers, vocab_size, max_len, dropout, true),
+            transformer: TransformerEncoder::new(&(p / "transformer"), n_embd, n_head, n_layers, vocab_size, positional_encoding, max_len, dropout, true),
             head: Linear::new(&(p / "lm_head"), n_embd, vocab_size)
         }
     }
@@ -369,8 +424,8 @@ impl NNModule for LanguageModel {
 }
 
 impl ModuleCopy for LanguageModel {
-    fn copy(&mut self, source: &Self) {
-        self.transformer.copy(&source.transformer);
-        self.head.copy(&source.head);
+    fn copy(&mut self, source: &Self) -> Result<(), WeightCopyError> {
+        self.transformer.copy(&source.transformer)?;
+        self.head.copy(&source.head)
     }
 }
\ No newline at end of file
